Rails.application.class.routes.draw do
  # Mercury::Engine.routes
  if !TAWORK_CONFIG['custom_devise_routes']
    devise_for :users, controllers: { omniauth_callbacks: "users/omniauth_callbacks" }
  end

  root 'home#index'
  get 'sink/index' #=> "sink#index"
  get 'sink/components' #=> "sink#components"

  get 'search/mentions' => "search#mentions"
  get 'search' => "search#index"

  get 'wiki' => "wiki/pages#index"
  namespace :work do
    resources :users, only: [:index, :new, :create] do
      member do
        get :reset_password_email
      end
    end
  end

  namespace :wiki do
    resources :pages do 
      member do
        get :subpages_dropdown
        get :combined
        get :history
        get :compare
        post :reorder
        post :star
        post :add_tag
        post :remove_tag

        post :attach
      end
    end
    resources :spaces do
      member do
        get :space_list
      end
    end
  end

  resources :attachments, only: [:show, :destroy] do
    member do
      post :upload
    end
  end
  resources :projects

  resources :tickets do
    resources :assignments
    collection do 
      get :user_search
    end

    member do
      post :trigger_event
      post :reorder
      patch :assignment
    end
  end

  resources :comments, only: [:create]
  # The priority is based upon order of creation: first created -> highest priority.
  # See how all your routes lay out with "rake routes".

  # You can have the root of your site routed with "root"
  # root 'welcome#index'

  # Example of regular route:
  #   get 'products/:id' => 'catalog#view'

  # Example of named route that can be invoked with purchase_url(id: product.id)
  #   get 'products/:id/purchase' => 'catalog#purchase', as: :purchase

  # Example resource route (maps HTTP verbs to controller actions automatically):
  #   resources :products

  # Example resource route with options:
  #   resources :products do
  #     member do
  #       get 'short'
  #       post 'toggle'
  #     end
  #
  #     collection do
  #       get 'sold'
  #     end
  #   end

  # Example resource route with sub-resources:
  #   resources :products do
  #     resources :comments, :sales
  #     resource :seller
  #   end

  # Example resource route with more complex sub-resources:
  #   resources :products do
  #     resources :comments
  #     resources :sales do
  #       get 'recent', on: :collection
  #     end
  #   end

  # Example resource route with concerns:
  #   concern :toggleable do
  #     post 'toggle'
  #   end
  #   resources :posts, concerns: :toggleable
  #   resources :photos, concerns: :toggleable

  # Example resource route within a namespace:
  #   namespace :admin do
  #     # Directs /admin/products/* to Admin::ProductsController
  #     # (app/controllers/admin/products_controller.rb)
  #     resources :products
  #   end
end
