class Searcher
  attr_reader :results

  def initialize options = {}
    @options = options

    @query = options[:query]
    @q = options[:q]

    @search_group = options[:search_group] || "pages,attachments"
    @type = options[:type]
    @comparator = options[:comparator] || "AND"
  end

  def self.search options = {}
    searcher = Searcher.new options
    searcher.search

    searcher
  end

  def search
    q = @query

    if @q
      q = @q
    else
      q = q.split(" ").join(" #{@comparator} ")
    end

    if @type
      q = "(#{q}) AND type:#{@type}"
    end

    client = Page.__elasticsearch__.client
    query = client.search index: search_group, 
      size: 30,
      q: q

    @results = Hashie::Mash.new query['hits']
  end

  def results_with_hits 
    grouped_results = {}

    hits = @results.hits
    hits.group_by(&:_type).each do |type, values|
      klass = type.classify.constantize
      ids = values.collect{|value| value._source.id }
      grouped_results[type] = klass.find(ids).group_by(&:id)
    end

    hits.each do |hit|
      yield grouped_results[hit._type][hit._source.id].first, hit
    end
  end

  def search_group
    @search_group
  end

  class Filters
    def self.ngram
      {
        type: "edgeNGram",
        min_gram: 2,
        max_gram: 20,
        token_chars: [
          "letter",
          "digit",
          "punctuation",
          "symbol"
        ]
      }
    end
  end

  class Analyzers

    def self.ngram
      {
        type: "custom",
        tokenizer: "whitespace",
        filter: [
          "lowercase",
          "asciifolding",
          "nGram_filter"
        ]
      }
    end

    def self.email_ngram
      {
        type: "custom",
        tokenizer: "uax_url_email",
        filter: [
          "lowercase",
          "asciifolding",
          "nGram_filter"
        ]
      }
    end

    def self.whitespace
      {
        type: "custom",
        tokenizer: "whitespace",
        filter: [
          "lowercase",
          "asciifolding"
        ]
      }
    end
  end
end
